local SpawnPrefab = GLOBAL.SpawnPrefab
local GetClock = GLOBAL.GetClock
local PIGSHOP_SHELVES = {
	--Taken from: prefabs/pig_shop.lua [DLC0003]
	"shelves_midcentury",
	"shelves_floating",
	"shelves_glass",
	"shelves_hutch",
	"shelves_marble",
	"shelves_wood",
	"shelves_fridge",
	"shelves_displaycase_metal",
	"shelves_metal"	
}
local SHELF_DATA = {
	--Taken from: prefabs/pig_shop.lua [DLC0003]
	["pig_shop_academy"] = {
		slots = {1, 5, 6},
		items = {"trinket_1", "trinket_2", "trinket_3"}
	},
	["pig_shop_antiquities"] = {
		slots = {1, 5, 6},
		items = {"trinket_1", "trinket_2", "trinket_3"}
	},
	["pig_shop_hatshop"] = {
		slots = {1, 2, 3, 4, 5, 6},
		items = {"petals", "petals", "cutgrass"} --4:2 => 2:1
	},
	["pig_shop_weapons"] = {
		slots = {5, 6, 3, 4},
		items = {"twigs"}
	},
	["pig_shop_arcane"] = {
		slots = {1, 5, 6},
		items = {"trinket_1", "trinket_2", "trinket_3"}
	},
	["pig_shop_florist"] = {
		slots = {3, 4, 5, 6},
		items = {"seeds"}
	},
	["pig_shop_hoofspa"] = {
		slots = {3, 4, 5, 6},
		items = {"petals"}
	},
	["pig_shop_general"] = {
		slots = {3, 4, 5, 6},
		items = {"rocks", "cutgrass"}
	},
	["pig_shop_deli"] = {
		slots = {1, 2, 3, 4, 5, 6},
		items = {"fish_raw_small", "bird_egg", "froglegs"} --2:2:2 => 1:1:1
	},
	["pig_shop_bank"] = {
		slots = {1, 2, 3},
		items = {"flint", "rocks", "rocks", "rocks", "nitre", "charcoal"} --2:6:2:2 => 1:3:1:1
	},
	["pig_shop_tinker"] = {
		slots = {3, 4, 5, 6},
		items = {"charcoal", "nitre", "papyrus", "charcoal"}
	}
}
local REFRESH_DAYS = GetModConfigData("REFRESH_DAYS")
local REFRESH_PEDESTALS = GetModConfigData("REFRESH_PEDESTALS")
local REFRESH_SHELVES = GetModConfigData("REFRESH_SHELVES")
local REFRESH_TRINKETS = GetModConfigData("REFRESH_TRINKETS")

local function NeedsRefresh()
	return REFRESH_DAYS == 1 or GetClock():GetNumCycles() % REFRESH_DAYS == 0
end
local function ShopBuyerPrefabPostInit(shopInstance)
	shopInstance:ListenForEvent("daycomplete",
		function()
			local saleItemData = shopInstance.saleitem
			if NeedsRefresh() and (not(saleItemData) or saleItemData[1] ~= "deed") then
				shopInstance:SoldItem()
			end
		end,
	GLOBAL.GetWorld())
end
local function strStartsWith(str, value)
	return str:sub(1, #value) == value
end
local function intStrLength(value)
	local absValue = value --math.abs(value)
	local length = 1 + math.floor(math.log10(absValue))
	--[[
	if value < 0 then
		length = length + 1
	end
	--]]
	return length
end
local function GetShelfData(interior)
	local interiorName = interior.dungeon_name
	--print("[DailyPigShopRefresh] currInteriorName = " .. interiorName)
	if strStartsWith(interiorName, "pig_shop") then
		--print("[DailyPigShopRefresh] " .. interiorName .. " is a pig shop!")
		local shopName = interiorName:sub(1, #interiorName - intStrLength(interior.unique_name)) --Removes the numbers at the end
		return SHELF_DATA[shopName]
	end
	return nil
end
local function RefreshShelfItems(shelfInstance)
	print("[DailyPigShopRefresh] Refreshing " .. shelfInstance.prefab .. "!")
	local interiorSpawner = GLOBAL.GetWorld().components.interiorspawner
	local shelfData = GetShelfData(interiorSpawner.current_interior)
	if shelfData and (REFRESH_TRINKETS or shelfData.items[1] ~= "trinket_1") then
		local shelfContainer = shelfInstance.components.container
		shelfContainer:DestroyContents()
		for currIndex, currSlot in ipairs(shelfData.slots) do
			local itemIndex = math.random(1, #shelfData.items)
			local item = SpawnPrefab(shelfData.items[itemIndex])
			if item then
				shelfContainer:GiveItem(item, currSlot)
			end
		end
		print("[DailyPigShopRefresh] Successfully refreshed " .. shelfInstance.prefab .. "!")
		shelfInstance.needsRefresh = false
	end
end
local function ShopShelfPrefabPostInit(shelfInstance)
	local oldReturnToInteriorScene = shelfInstance.returntointeriorscene
	shelfInstance:ListenForEvent("daycomplete",
		function()
			if NeedsRefresh() then
				print("[DailyPigShopRefresh] " .. shelfInstance.prefab .. " needs refreshing!")
				if shelfInstance.inlimbo then --Is it loaded in a different interior?
					--Yes it is! We'll update it next time he enters the shop
					shelfInstance.needsRefresh = true
				else
					--No it is not... We can just update it right now!
					RefreshShelfItems(shelfInstance)
				end
			end
		end,
	GLOBAL.GetWorld())
	
	shelfInstance.returntointeriorscene = function(inst) --Override prefabs/shelf.lua -> returntointeriorscene() [DLC0003]
		local world = GLOBAL.GetWorld()
		oldReturnToInteriorScene(inst)
		if world and inst.needsRefresh then
			inst:DoTaskInTime(0, function()
				RefreshShelfItems(inst)
			end)
		end
	end
end

if REFRESH_PEDESTALS then
	print("[DailyPigShopRefresh] Registering ShopBuyerPrefabPostInit")
	AddPrefabPostInit("shop_buyer", ShopBuyerPrefabPostInit)
end
if REFRESH_SHELVES then
	print("[DailyPigShopRefresh] Registering ShopShelfPrefabPostInit")
	for currIndex, currShelfPrefab in ipairs(PIGSHOP_SHELVES) do
		AddPrefabPostInit(currShelfPrefab, ShopShelfPrefabPostInit)
	end
end
